#!/usr/bin/env python
# coding: utf8
#
#

import time
import gtk
import gobject
import pango

from lm75 import read_lm75
from hyt221 import read_hyt221
from bmp085 import *

class WeatherStation:
  def __init__(self, title, font, bgcolor):

    # create gui
    self.window = gtk.Window()

    self.window.connect("delete_event", self.delete_event)
    self.window.connect('destroy', self.destroy)

    self.window.set_title(title)
#    self.window.set_border_width(10)
    main_vbox = gtk.VBox(False, 0)
    self.window.add(main_vbox)
    
    # temperatur frame
    frame_temperatur = gtk.Frame("Temperatur")
    frame_temperatur.set_border_width(10)
    main_vbox.pack_start(frame_temperatur, True, True, 0)
    self.label_temperatur = gtk.Label("---- °C")
    self.label_temperatur.modify_font(pango.FontDescription(font))
    ebox_temperatur = gtk.EventBox()
    ebox_temperatur.add(self.label_temperatur)
    ebox_temperatur.set_border_width(10)
    ebox_temperatur.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(bgcolor))
    frame_temperatur.add(ebox_temperatur)

    # pressure frame
    frame_pressure = gtk.Frame("Luftdruck")
    frame_pressure.set_border_width(10)
    main_vbox.pack_start(frame_pressure, True, True, 0)
    self.label_pressure = gtk.Label("---- hPa")
    self.label_pressure.modify_font(pango.FontDescription(font))
    ebox_pressure = gtk.EventBox()
    ebox_pressure.add(self.label_pressure)
    ebox_pressure.set_border_width(10)
    ebox_pressure.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(bgcolor))
    frame_pressure.add(ebox_pressure)

    # humidity frame
    frame_humidity = gtk.Frame("Feuchte")
    frame_humidity.set_border_width(10)
    main_vbox.pack_start(frame_humidity, True, True, 0)
    self.label_humidity = gtk.Label("---- %")
    self.label_humidity.modify_font(pango.FontDescription(font))
    ebox_humidity = gtk.EventBox()
    ebox_humidity.add(self.label_humidity)
    ebox_humidity.set_border_width(10)
    ebox_humidity.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(bgcolor))
    frame_humidity.add(ebox_humidity)

    # add statusbar
    self.statusbar = gtk.Statusbar()
    main_vbox.pack_start(self.statusbar, False, False, 0)
    self.statusbar_context_id = self.statusbar.get_context_id("Statusbar")
 
    self.window.show_all()


    # register periodic timer to update values
    self.counter = 0
    gobject.timeout_add_seconds(1, self.callback)

  def delete_event(self, widget, event, data=None):
    print "delete event occurred"
    return False

  def destroy(self, widget, data=None):
    print "quit event occurred"
    gtk.main_quit()


  def callback(self):
    if self.counter >= 1:
      self.statusbar.pop(self.statusbar_context_id)
    if self.counter >= 5:
      self.statusbar.push(self.statusbar_context_id, 'Read sensors')
      # read lm75
      #self.label_temperatur.set_text("%0.1f °C" % read_lm75())
      # read bmp085
      data = read_bmp085(1, 0x77, 0)
      self.label_temperatur.set_text("%0.1f °C" % data['temperature'])
      self.label_pressure.set_text("%0.1f hPa" % (data['pressure']/100)) 
      # read hyt221
      data = read_hyt221(1, 0x28)
      self.label_humidity.set_text("%.1f %%" % data['humidity'])
      # reset counter
      self.counter = 0
    self.counter += 1
    return True


if __name__ == '__main__':
  weatherstation = WeatherStation("Wetterstation", "Sans Italic 64", "#f9fff9")
  gtk.main()

